
      program test

      implicit none

      include "mpif.h"
      include "pfft.f"
      include "fftw3.f"
      
      integer np(3), ierror, errorcode, comm_cart_2d
      integer(ptrdiff_t_kind) n(3)
      integer(ptrdiff_t_kind) :: alloc_local_gc, alloc_local
      integer(ptrdiff_t_kind) :: gcells_below(3), gcells_above(3)
      integer(ptrdiff_t_kind) :: local_ngc(3), local_gc_start(3)
      integer(ptrdiff_t_kind) :: local_ni(3), local_i_start(3)
      integer(ptrdiff_t_kind) :: local_no(3), local_o_start(3)
      integer(8) fftplan, gcplan
      complex(8), allocatable ::  data_in(:)
      complex(8), allocatable ::  data_out(:)
      
      n = (/ 2,2,4 /)
      np = (/ 1,2,2 /)
      gcells_below = (/ 0,0,0 /)
      gcells_above = (/ 1,0,0 /)

!     Initialize MPI and PFFT
      call MPI_Init(ierror)
      call dpfft_init();
      
!     Create two-dimensional process grid of
!     size np(1) x np(2), if possible
      call dpfft_create_procmesh_2d(ierror, MPI_COMM_WORLD, &
           & np(2), np(3), comm_cart_2d)
      if (ierror .ne. 0) then
         call message("Error: This test file only works with &
              & 4 processors.", MPI_COMM_WORLD)
         call MPI_Abort(MPI_COMM_WORLD, errorcode, ierror)
      endif
      
!     Get parameters of data distribution
      call dpfft_local_size_3d(alloc_local, n, &
           & comm_cart_2d, PFFT_TRANSPOSED, &
           & local_ni, local_i_start, local_no, local_o_start)
      
!     Get parameters of ghostcell distribution
      call dpfft_local_size_gcells_3d(alloc_local_gc, &
           & local_no, local_o_start, alloc_local, &
           & gcells_below, gcells_above, &
           & local_ngc, local_gc_start)
      
!     Allocate memory
      allocate(data_in(alloc_local))
      allocate(data_out(alloc_local_gc))
!       data_in  = reshape(data_in,  (/ local_ni(1), local_ni(2), local_ni(3) /))
!       data_out = reshape(data_out, (/ local_no(1), local_no(2), local_no(3) /))
      
!     Plan parallel forward FFT
      call dpfft_plan_dft_3d(fftplan, n, &
           & data_in, data_out, comm_cart_2d, &
           & FFTW_FORWARD, PFFT_TRANSPOSED + PFFT_FORWARD, FFTW_MEASURE)

!     Plan ghostcells
      call dpfft_plan_gcells_3d(gcplan, n, gcells_below, gcells_above, &
           & data_out, comm_cart_2d, PFFT_TRANSPOSED + PFFT_GC_AFTER_FFT)
      
!     Initialize input with random numbers
      call dpfft_init_input_3d(n, local_ni, local_i_start, &
           & comm_cart_2d, data_in)

!     Print input data
      call dpfft_apr_complex_3d(data_in, local_ni, local_i_start, &
           & "PFFT, g_hat"//CHAR(0), MPI_COMM_WORLD)

!     execute parallel forward FFT
      call dpfft_execute(fftplan)

!     Print transformed data 
      call dpfft_apr_complex_permuted_3d( &
           & data_out, local_no, local_o_start, 3, 2, 1, &
           & "PFFT, g"//CHAR(0), MPI_COMM_WORLD)

!     send ghostcells
      call dpfft_exchange(gcplan)

!     Print transformed data with ghost cells
      call dpfft_apr_complex_permuted_3d( &
           & data_out, local_ngc, local_gc_start, 3, 2, 1, &
           & "PFFT, g with gcells"//CHAR(0), MPI_COMM_WORLD)
      
!     free mem and finalize
      call dpfft_destroy_plan(fftplan)
      call dpfft_destroy_gcplan(gcplan)
      call MPI_Comm_free(comm_cart_2d, ierror)
      deallocate(data_out)
      deallocate(data_in)
  
!     Finalize MPI
      call dpfft_cleanup()
      call MPI_Finalize(ierror)
      end
      
      
      
      subroutine message(string, comm)
      
      integer comm, myrank, ierror
      character string*(*)
      
      myrank = 0
      call MPI_Comm_rank(comm, myrank, ierror);
      if(myrank .eq. 0) then
        write(*,*) string(:)
      endif
      end subroutine

