/*
 * Copyright (c) 2010 Michael Pippig
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "pfft.h"
#include "ipfft.h"


/****************************************
 *  some implemented pfft_profiles
 ****************************************/

/*  n0/np0 x n1/np1 x n2  to  N0/np1 x N2/np0 x N1  */
void PX(profile_n021_np102)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to n0/np0 x n1/np1 x N2 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_NONE;
  
  /* remap (n0/np0 x n1/np1) x N2 to (n0 x n1/np1) x N2/np0 */
  prf->first_dims[0] = 2;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_OUT;
  
  /* transform n0 x n1/np1 x N2/np0 to N0 x N2/np0 x n1/np1 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_021;
  
  /* remap (N0 x N2/np0) x n1/np1 to (N0/np1 x N2/np0) x n1 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N0/np1 x N2/np0 x n1 to N0/np1 x N2/np0 x N1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_NONE;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_021) */


/*  n0/np0 x n1/np1 x n2  to  N0/np1 x N1/np0 x N2  */
void PX(profile_n012_np102)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to n1/np1 x n0/np0 x N2 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_102;
  
  /* remap (n1/np1 x n0/np0) x N2 to (n1 x n0/np0) x N2/np1 */
  prf->first_dims[0] = 2;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_OUT;
  
  /* transform n1 x n0/np0 x N2/np1 to N1 x n0/np0 x N2/np1 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_NONE;
  
  /* remap N1 x (n0/np0 x N2/np1) to N1/np0 x (n0 x N2/np1) */
  prf->first_dims[1] = 1;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N1/np0 x n0 x N2/np1 to N0 x N1/np0 x N2/np1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_102;
  
  /* remap (N0 x N1/np0) x N2/np1 to (N0/np1 x N1/np0) x N2 */
  prf->first_dims[2] = 2;
  prf->remap_flag[2] = FFTW_MPI_TRANSPOSED_IN;
} /* end PX(profile_012) */


/*  n0/np0 x n1/np1 x n2  to  N1/np0 x N2/np1 x N0  */
void PX(profile_n120_np012)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to n1/np1 x n0/np0 x N2 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_102;
  
  /* remap (n1/np1 x n0/np0) x N2 to (n1 x n0/np0 x N2/np1 */
  prf->first_dims[0] = 2;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_OUT;
  
  /* transform n1 x n0/np0 x N2/np1 to N1 x N2/np1 x n0/np0 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_021;
  
  /* remap (N1 x N2/np1) x n0/np0 to (N1/np0 x N2/np1) x n0 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N1/np0 x N2/np1 x n0 to N1/np0 x N2/np1 x N0 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_NONE;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_120) */


/*  n0/np0 x n1/np1 x n2  to  N2/np0 x N0/np1 x N1  */
void PX(profile_n201_np012)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to n0/np0 x n1/np1 x N2 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_NONE;
  
  /* remap (n0/np0 x n1/np1) x N2 to (n0 x n1/np1) x N2/np0 */
  prf->first_dims[0] = 2;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_OUT;
  
  /* transform n0 x n1/np1 x N2/np0 to N0 x N2/np0 x n1/np1 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_021;
  
  /* remap (N0 x N2/np0) x n1/np1 to (N0/np1 x N2/np0) x n1 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N0/np1 x N2/np0 x n1 to N2/np0 x N0/np1 x N1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_102;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_201) */


/*  n0/np0 x n1/np1 x n2  to  N1/np0 x N2/np1 x N0  */
void PX(profile_n120_np012_IN)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap (N2 x n0/np0) x n1/np1 to (N2/np1 x n0/np0) x n1 */
  prf->first_dims[0] = 2;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N2/np1 x n0/np0 x n1 to N1 x N2/np1 x n0/np0 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_201;
  
  /* remap (N1 x N2/np1) x n0/np0  to (N1/np0 x N2/np1) x n0 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N1/np0 x N2/np1 x n0 to N1/np0 x N2/np1 x N0 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_NONE;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_120_IN) */


/* n0/np0 x n1/np1 x n2  to  N0/np1 x N2/np0 x N1  */
/* forw and back profile is the same */
void PX(profile_n021_np102_outofplace)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap (N2 x n0/np0) x n1/np1 to (N2/np1 x n0/np0) x n1 */
  prf->first_dims[0] = 2;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N2/np1 x n0/np0 x n1 to N1 x N2/np1 x n0/np0 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_201;
  
  /* remap (N1 x N2/np1) x n0/np0 to (N1/np0 x N2/np1) x n0 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N1/np0 x N2/np1 x n0 to N1/np0 x N2/np1 x N0 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_NONE;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_021_outofplace) */


/*  n0/np0 x n1/np1 x n2  to  N2/np0 x N0/np1 x N1  */
void PX(profile_n201_np012_IN)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap N2 x (n0/np0 x n1/np1) to N2/np0 x (n0 x n1/np1) */
  prf->first_dims[0] = 1;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N2/np0 x n0 x n1/np1 to N0 x N2/np0 x n1/np1 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_102;
  
  /* remap (N0 x N2/np0) x n1/np1 to (N0/np1 x N2/np0) x n1 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N0/np1 x N2/np0 x n1 to N2/np0 x N0/np1 x N1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_102;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_201_IN) */


/*  n0/np0 x n1/np1 x n2  to  N2/np0 x N0/np1 x N1  */
void PX(profile_n201_np012_IN1)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap N2 x (n0/np0 x n1/np1) to (n0 x n1/np1) x N2/np0 */
  prf->first_dims[0] = 1;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN| FFTW_MPI_TRANSPOSED_OUT;
  
  /* transform n0 x n1/np1 x N2/np0 to N0 x N2/np0 x n1/np1 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_021;
  
  /* remap (N0 x N2/np0) x n1/np1 to (N0/np1 x N2/np0) x n1 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N0/np1 x N2/np0 x n1 to N2/np0 x N0/np1 x N1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_102;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_201_IN1) */


/*  n0/np0 x n1/np1 x n2  to  N2/np0 x N0/np1 x N1  */
void PX(profile_n201_np012_IN2)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap N2 x (n0/np0 x n1/np1) to (n0 x n1/np1) x N2/np0 */
  prf->first_dims[0] = 1;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN| FFTW_MPI_TRANSPOSED_OUT;
  
  /* transform n0 x n1/np1 x N2/np0 to N0 x n1/np1 x N2/np0 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_NONE;
  
  /* remap N0 x (n1/np1 x N2/np0) to (n1 x N2/np0) x N0/np1 */
  prf->first_dims[1] = 1;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN| FFTW_MPI_TRANSPOSED_OUT;
  
  /* transform n1 x N2/np0 x N0/np1 to N2/np0 x N0/np1 x N1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_120;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_201_IN2) */


/*  n0/np0 x n1/np1 x n2  to  N2/np1 x N1/np0 x N0  */
/*  self inverse  */
void PX(profile_n210_np102)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap (N2 x n0/np0) x n1/np1 to (N2/np1 x n0/np0) x n1 */
  prf->first_dims[0] = 2;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N2/np1 x n0/np0 x n1 to N1 x N2/np1 x n0/np0 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_201;
  
  /* remap (N1 x N2/np1) x n0/np0  to (N1/np0 x N2/np1) x n0 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N1/np0 x N2/np1 x n0 to N2/np1 x N1/np0 x N0 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_102;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_n210_np10x) */


/*  n0/np0 x n1/np1 x n2  to  N0/np1 x N2/np0 x N1  */
/*  only out of place because of middle 1dFFT  */
/*  self inverse  */
void PX(profile_n021_np102_outofplace2)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap N2 x (n0/np0 x n1/np1) to N2/np0 x (n0 x n1/np1) */
  prf->first_dims[0] = 1;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N2/np0 x n0 x n1/np1 to N0 x n1/np1 x N2/np0 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_120;
  
  /* remap N0 x (n1/np1 x N2/np0) to N0/np1 x (n1 x N2/np0) */
  prf->first_dims[1] = 1;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N0/np1 x n1 x N2/np0 to N0/np1 x N2/np0 x N1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_021;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_n021_np10x) */

/* change middle 1dFFT to enable inplace transformation */
/*  n0/np0 x n1/np1 x n2  to  N0/np1 x N2/np0 x N1  */
/*  self inverse  */
void PX(profile_n021_np102_inplace)(PX(profile) *prf){
  /* transform n0/np0 x n1/np1 x n2 to N2 x n0/np0 x n1/np1 */
  prf->fft_flag[0] = PFFT_TRANSPOSE_201;
  
  /* remap N2 x (n0/np0 x n1/np1) to N2/np0 x (n0 x n1/np1) */
  prf->first_dims[0] = 1;
  prf->remap_flag[0] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N2/np0 x n0 x n1/np1 to N0 x N2/np0 x n1/np1 */
  prf->fft_flag[1] = PFFT_TRANSPOSE_102;
  
  /* remap (N0 x N2/np0) x n1/np1 to (N0/np1 x N2/np0) x n1 */
  prf->first_dims[1] = 2;
  prf->remap_flag[1] = FFTW_MPI_TRANSPOSED_IN;
  
  /* transform N0/np1 x N2/np0 x n1 to N0/np1 x N2/np0 x N1 */
  prf->fft_flag[2] = PFFT_TRANSPOSE_NONE;
  
  /* omit last remap */
  prf->first_dims[2] = 0;
  prf->remap_flag[2] = 0;
} /* end PX(profile_n021_np10x_inplace) */

