#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <complex.h>

#include <fftw3.h>
#include <fftw3-mpi.h>
#include <pfft.h>

int main(int argc, char **argv)
{
  int np[2];
  ptrdiff_t n[3];
  ptrdiff_t alloc_local;
  ptrdiff_t local_ni[3], local_i_start[3];
  ptrdiff_t local_no[3], local_o_start[3];
  fftw_complex *data_in, *data_out;
  pfft_plan fftplan_forw=NULL, fftplan_back=NULL;
  MPI_Comm comm_cart_2d;
  
  n[0] = 2; n[1] = 2; n[2] = 4;
  np[0] = 2; np[1] = 2;
  
  /* Initialize MPI and PFFT */
  MPI_Init(&argc, &argv);
  pfft_init();

  /* Create two-dimensional process grid of size np[0] x np[1], if possible */
  if( pfft_create_procmesh_2d(MPI_COMM_WORLD, np[0], np[1], &comm_cart_2d) ){
    pfft_fprintf(MPI_COMM_WORLD, stderr, "Error: This test file only works with 4 processors.\n");
    return 1;
  }

  /* Get parameters of data distribution */
  alloc_local = pfft_local_size_3d(n, comm_cart_2d, PFFT_TRANSPOSED,
      local_ni, local_i_start, local_no, local_o_start);

  /* Allocate memory */
  data_in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*alloc_local);
  data_out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*alloc_local);

  /* Plan parallel forward FFT */
  fftplan_forw = pfft_plan_dft_3d(
      n, data_in, data_out, comm_cart_2d, FFTW_FORWARD, PFFT_TRANSPOSED | PFFT_FORWARD, FFTW_MEASURE);
  
  /* Plan parallel backward FFT */
  fftplan_back = pfft_plan_dft_3d(
      n, data_out, data_in, comm_cart_2d, FFTW_BACKWARD, PFFT_TRANSPOSED | PFFT_BACKWARD, FFTW_MEASURE);

  /* Initialize input with random numbers */
  pfft_init_input_3d(n, local_ni, local_i_start, comm_cart_2d,
      data_in);

  /* Print input data */
  pfft_apr_complex_3d(
      data_in, local_ni, local_i_start,
      "PFFT, g_hat", MPI_COMM_WORLD);

  /* execute parallel forward FFT */
  pfft_execute(fftplan_forw);
  
  /* Print transformed data */
  pfft_apr_complex_permuted_3d(
      data_out, local_no, local_o_start, 2, 1, 0,
      "PFFT, g", MPI_COMM_WORLD);
  
  /* execute parallel backward FFT */
  pfft_execute(fftplan_back);
  
  /* Scale data */
  for(ptrdiff_t l=0; l < local_ni[0] * local_ni[1] * local_ni[2]; l++)
    data_in[l] /= (n[0]*n[1]*n[2]);
  
  /* Print back transformed data */
  pfft_apr_complex_3d(
      data_in, local_ni, local_i_start,
      "PFFT^H, g_hat", MPI_COMM_WORLD);
  
  /* free mem and finalize */
  pfft_destroy_plan(fftplan_forw);
  pfft_destroy_plan(fftplan_back);
  MPI_Comm_free(&comm_cart_2d);
  fftw_free(data_out);
  fftw_free(data_in);
  
  /* Finalize MPI */
  MPI_Finalize();
  return 0;
}
